package cn.tonyandmoney.tuon.qywx.controller


import cn.tonyandmoney.tuon.core.error.BaseResp
import cn.tonyandmoney.tuon.core.error.OpException
import cn.tonyandmoney.tuon.core.error.SessionNoUserException
import cn.tonyandmoney.tuon.core.session.SessionUtils
import cn.tonyandmoney.tuon.core.user.IUser
import cn.tonyandmoney.tuon.qywx.WxErrorCode
import cn.tonyandmoney.tuon.qywx.bean.WxUser
import cn.tonyandmoney.tuon.qywx.service.IQywxService
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.server.ServerWebExchange
import reactor.core.publisher.Mono
import reactor.core.publisher.onErrorResume
import java.util.function.Function

@RestController
@RequestMapping("/qywx")
class QywxController {

    @Autowired
    private lateinit var qywxService: IQywxService

    @GetMapping("/user")
    fun getUserInfo(@RequestParam("code", required = false) code: String?,
                    @RequestParam("agentId") agentId: String,
                    exchange: ServerWebExchange): Mono<BaseResp<IUser>> {

        return SessionUtils.getUser<WxUser>(exchange)
                .onErrorResume(SessionNoUserException::class.java) {
                    if (code.isNullOrBlank()) {
                        Mono.error(OpException(WxErrorCode.ERR_CODE))
                    } else {
                        qywxService.getUserInfo(agentId, code)
                                .flatMap { user -> qywxService.getUserById(agentId, user.id) }
                                .flatMap { info -> SessionUtils.setUser(exchange, info) }
                    }
                }
                .flatMap { info -> Mono.just(BaseResp.ok(info)) }
                .onErrorResume {
                    if (it is OpException) {
                        Mono.just(it.toResp())
                    } else {
                        Mono.just(BaseResp.error(it))
                    }
                }


    }

}
